@@ -1,49 +1,43 @@
-// $Id: RadixSet.cc 1.10 Wed, 11 Feb 1998 16:54:38 -0800 cengiz $
-// 
+//  $Id: RadixSet.cc 1.21 Thu, 14 Jan 1999 14:56:48 -0800 cengiz $
+//
 //  Copyright (c) 1994 by the University of Southern California
-//  and/or the International Business Machines Corporation.
 //  All rights reserved.
 //
-//  Permission to use, copy, modify, and distribute this software and
-//  its documentation in source and binary forms for lawful
-//  non-commercial purposes and without fee is hereby granted, provided
-//  that the above copyright notice appear in all copies and that both
-//  the copyright notice and this permission notice appear in supporting
-//  documentation, and that any documentation, advertising materials,
-//  and other materials related to such distribution and use acknowledge
-//  that the software was developed by the University of Southern
-//  California, Information Sciences Institute and/or the International
-//  Business Machines Corporation.  The name of the USC or IBM may not
-//  be used to endorse or promote products derived from this software
-//  without specific prior written permission.
+//  Permission to use, copy, modify, and distribute this software and its
+//  documentation in source and binary forms for lawful non-commercial
+//  purposes and without fee is hereby granted, provided that the above
+//  copyright notice appear in all copies and that both the copyright
+//  notice and this permission notice appear in supporting documentation,
+//  and that any documentation, advertising materials, and other materials
+//  related to such distribution and use acknowledge that the software was
+//  developed by the University of Southern California, Information
+//  Sciences Institute. The name of the USC may not be used to endorse or
+//  promote products derived from this software without specific prior
+//  written permission.
 //
-//  NEITHER THE UNIVERSITY OF SOUTHERN CALIFORNIA NOR INTERNATIONAL
-//  BUSINESS MACHINES CORPORATION MAKES ANY REPRESENTATIONS ABOUT
-//  THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
-//  PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
-//  INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
-//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND 
-//  NON-INFRINGEMENT.
+//  THE UNIVERSITY OF SOUTHERN CALIFORNIA DOES NOT MAKE ANY
+//  REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
+//  PURPOSE.  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
+//  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
+//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
+//  TITLE, AND NON-INFRINGEMENT.
 //
-//  IN NO EVENT SHALL USC, IBM, OR ANY OTHER CONTRIBUTOR BE LIABLE FOR ANY
-//  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, WHETHER IN CONTRACT,
-//  TORT, OR OTHER FORM OF ACTION, ARISING OUT OF OR IN CONNECTION WITH,
-//  THE USE OR PERFORMANCE OF THIS SOFTWARE.
+//  IN NO EVENT SHALL USC, OR ANY OTHER CONTRIBUTOR BE LIABLE FOR ANY
+//  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, WHETHER IN CONTRACT, TORT,
+//  OR OTHER FORM OF ACTION, ARISING OUT OF OR IN CONNECTION WITH, THE USE
+//  OR PERFORMANCE OF THIS SOFTWARE.
 //
 //  Questions concerning this software should be directed to 
-//  info-ra@isi.edu.
+//  ratoolset@isi.edu.
 //
-//  Author(s): Cengiz Alaettinoglu <cengiz@isi.edu>
+//  Author(s): Cengiz Alaettinoglu <cengiz@ISI.EDU>
 
-#include "config.hh"
+#include "config.h"
 
 #include <cassert>
 #include <cstdio>
 #include <iostream.h>
 #include <iomanip.h>
-#include "dbase/Prefask.h"
-
-#define DEBUG
 
 #include "RadixSet.hh"
 
@@ -74,15 +68,16 @@ u_int64_t bits[] = { 0x100000000ULL,
 };
 
 FixedSizeAllocator RadixTreeAllocator(sizeof(RadixTree), 1000);
+bool RadixSet::compressedPrint = false;
 
 inline const RadixTree* RadixTree::Iterator::first() {
    dfsStack.clear();
-   dfsStack.push(NULL);
+   dfsStack.push((RadixTree *) NULL);
 
    last = root;
 
    if (!last)
-      return NULL;
+      return (const RadixTree *) NULL;
 
    foreachchild(c)
       if (last->chld[c])
@@ -97,7 +92,7 @@ inline const RadixTree* RadixTree::Iterator::next(const RadixTree* _last) {
    last = dfsStack.pop();
 
    if (!last)
-      return NULL;
+      return (const RadixTree *) NULL;
 
    foreachchild(c)
       if (last->chld[c])
@@ -193,12 +188,12 @@ bool RadixSet::SortedPrefixIterator::first(u_int &_addr, u_int &_leng) {
       // but since radix tree is almost sorted, it will be around (O(n))
       for (p = l.head(); 
 	   p && (p->addr < addr || (p->addr == addr && p->leng < leng)); 
-	   p = l.next(p->l))
+	   p = l.next(p))
 	 ;
       if (p)
-	 l.insert_before(p->l, (new PrefixLNode(addr, leng))->l);
+	 l.insertBefore(p, (new PrefixLNode(addr, leng)));
       else
-	 l.append((new PrefixLNode(addr, leng))->l);
+	 l.append((new PrefixLNode(addr, leng)));
    }
 
    return next(_addr, _leng);
@@ -212,7 +207,7 @@ bool RadixSet::SortedPrefixIterator::next(u_int &_addr, u_int &_leng) {
 
    _addr = p->addr;
    _leng  = p->leng;
-   l.unlink(p->l);
+   l.remove(p);
    delete p;
 
    return true;
@@ -239,12 +234,12 @@ bool RadixSet::SortedPrefixRangeIterator::first(u_int &_addr, u_int &_leng,
 		 || (p->addr == addr && p->leng == leng && p->start == start
 		     && p->end < end)
 		 ); 
-	   p = l.next(p->l))
+	   p = l.next(p))
 	 ;
       if (p)
-	 l.insert_before(p->l, (new PrefixLNode(addr, leng, start, end))->l);
+	 l.insertBefore(p, new PrefixLNode(addr, leng, start, end));
       else
-	 l.append((new PrefixLNode(addr, leng, start, end))->l);
+	 l.append(new PrefixLNode(addr, leng, start, end));
    }
 
    return next(_addr, _leng, _start, _end);
@@ -261,7 +256,7 @@ bool RadixSet::SortedPrefixRangeIterator::next(u_int &_addr, u_int &_leng,
    _leng  = p->leng;
    _start = p->start;
    _end = p->end;
-   l.unlink(p->l);
+   l.remove(p);
    delete p;
 
    return true;
@@ -362,11 +357,11 @@ RadixTree *RadixTree::insert(u_int _addr, u_int _leng, u_int64_t _rngs) {
 
    RadixTree *root = this;
    Direction dir, pdir;
-   RadixTree *parent = NULL;
+   RadixTree *parent = (RadixTree *) NULL;
    RadixTree *now = this;
    pStack.push(parent);
 
-   RadixTree *cparent = NULL;
+   RadixTree *cparent = (RadixTree *) NULL;
    u_int caddr, cleng;
 
    while (_rngs) {
@@ -446,7 +441,7 @@ RadixTree *RadixTree::insert(u_int _addr, u_int _leng, u_int64_t _rngs) {
 		     if (! now->chld[c]->chld[gc]) {
 			tmp = now->chld[c];
 			now->chld[c] = tmp->chld[1-gc];
-			tmp->chld[1-gc] = NULL;
+			tmp->chld[1-gc] = (RadixTree *) NULL;
 			delete tmp;
 			break;
 		     }
@@ -487,7 +482,7 @@ bool RadixTree::contains(u_int _addr, u_int _leng, u_int64_t _rngs) const {
       case LEFT:
       case RIGHT:
 	 _rngs &= ~now->rngs;
-	 if (_rngs)
+	 if (! _rngs)
 	    return true;
 
 	 now = now->chld[dir];
@@ -508,7 +503,7 @@ RadixTree *RadixTree::remove(u_int _addr, u_int _leng, u_int64_t _rngs) {
    int pStackPos = pStack.getPosition();
    int dStackPos = dStack.getPosition();
 
-   pStack.push(NULL);
+   pStack.push((RadixTree *) NULL);
    dStack.push(HERE);
 
    u_int64_t extraRngs = 0;
@@ -564,7 +559,7 @@ RadixTree *RadixTree::remove(u_int _addr, u_int _leng, u_int64_t _rngs) {
 	    foreachchild(c)
 	       if (! now->chld[c]) {
 		  tmp = now->chld[1-c];
-		  now->chld[1-c] = NULL;
+		  now->chld[1-c] = (RadixTree *) NULL;
 		  delete now;
 		  now = tmp;
 		  break;
@@ -578,7 +573,7 @@ RadixTree *RadixTree::remove(u_int _addr, u_int _leng, u_int64_t _rngs) {
 
    if (now && !now->rngs && !now->chld[0] && !now->chld[1]) {
       delete now;
-      return NULL;
+      return (RadixTree *) NULL;
    }
 
    return now;
@@ -598,7 +593,7 @@ RadixTree *RadixTree::removeRange(u_int64_t _rngs) {
       foreachchild(c)
 	 if (! chld[c]) {
 	    tmp = chld[1-c];
-	    chld[1-c] = NULL;
+	    chld[1-c] = (RadixTree *) NULL;
 	    delete this;
 	    return tmp;
 	 }
@@ -614,22 +609,22 @@ RadixTree *RadixTree::and(const RadixTree *b) {
 
    if (!b) {
       delete this;
-      return NULL;
+      return (RadixTree *) NULL;
    }
 
    static BoundedStack<RadixTree*>       pStack(65);
    static BoundedStack<const RadixTree*> bStack(65);
    static BoundedStack<u_int>            rStack(65);
-   RadixTree *result = NULL;
+   RadixTree *result = (RadixTree *) NULL;
    RadixTree *now;
    const RadixTree *othr, *p;
    bool found;
    u_int64_t incRngs, leftRngs;
    Direction dir, rdir;
 
-   pStack.push(NULL);
+   pStack.push((RadixTree *) NULL);
    pStack.push(this);
-   bStack.push(NULL);
+   bStack.push((RadixTree *) NULL);
    bStack.push(b);
    rStack.push(0);
    rStack.push(0);
@@ -693,7 +688,7 @@ RadixTree *RadixTree::and(const RadixTree *b) {
 RadixTree *RadixTree::setminus(const RadixTree *b) {
    if (this == b) {
       delete this;
-      return NULL;
+      return (RadixTree *) NULL;
    }
 
    if (!b || !this)
@@ -804,26 +799,91 @@ bool RadixTree::equals(const RadixTree *b) const {
 ostream& operator<<(ostream& o, const RadixSet &set) {
    char buffer[64];
    bool need_comma = false;
-   u_int addr;
-   u_int leng;
+   u_int addr, leng, n, m;
 
    o << "{";
    if (set.root) {
-      RadixSet::PrefixIterator itr(&set);
-      for (bool flag = itr.first(addr, leng); 
-	   flag;
-	   flag = itr.next(addr, leng)) {
-	 if (need_comma)
-	    o << ", ";
-	 else
-	    need_comma = true;
-	 o << int2quad(buffer, addr) << "/" << leng;
+      if (RadixSet::compressedPrint) {
+	 RadixSet::PrefixRangeIterator itr(&set);
+	 for (bool flag = itr.first(addr, leng, n, m); 
+	      flag;
+	      flag = itr.next(addr, leng, n, m)) {
+	    if (need_comma)
+	       o << ", ";
+	    else
+	       need_comma = true;
+	    o << int2quad(buffer, addr) << "/" << leng << "^" << n << "-" << m;
+	 }
+      } else {
+	 RadixSet::PrefixIterator itr(&set);
+	 for (bool flag = itr.first(addr, leng); 
+	      flag;
+	      flag = itr.next(addr, leng)) {
+	    if (need_comma)
+	       o << ", ";
+	    else
+	       need_comma = true;
+	    o << int2quad(buffer, addr) << "/" << leng;
+	 }
       }
    }
    o << "}";
    return o;
 }
 
+// make tree include any more specifics, i.e. ^+, ^-, ^n-m, ^n (i.e. ^n-n)
+RadixTree *RadixTree::makeMoreSpecific(int code, int n, int m) {
+   u_int64_t _rngs;
+   int i;
+
+   if (!this)
+      return this;
+
+   if (rngs) {
+      switch (code) {
+      case 0: // ^-
+	 // clear first set bit make the following bits all 1's
+	 rngs = rngs >> 1;
+      case 1: // ^+
+	 // find first set bit make the following bits all 1's
+	 for (i = 0; i < 32; ++i)
+	    if (rngs & bits[i])
+	       break;
+	 rngs |= ~masks[i];
+	 break;
+      case 2: // ^n-m
+	 for (i = 0; i < 32; ++i)
+	    if (rngs & bits[i])
+	       break;
+	 if (i > m) {
+	    rngs = 0;
+	    break;
+	 }
+	 if (i > n)
+	    n = i;
+	 if (n == 0)
+	    n = 1;
+	 rngs = masks[m] & ~masks[n-1];
+      }
+
+      // delete subtree of now that is included in already in _rngs
+      if (rngs)
+	 foreachchild(c)
+	    chld[c] = chld[c]->removeRange(rngs);
+   }
+
+   foreachchild(c)
+      if (chld[c])
+	 chld[c] = chld[c]->makeMoreSpecific(code, n, m);
+
+   if (!chld[0] && !chld[1] && !rngs) {
+      delete this;
+      return (RadixTree *) NULL;
+   }
+
+   return this;
+}
+
 #ifdef MAIN
 
 char* int2quad(char *buffer, unsigned int i) {
