@@ -1,4 +1,4 @@
-/* $Id: re2dfa.c 1.5.2.1 Mon, 12 Jan 1998 15:33:17 -0800 cengiz $
+/* $Id: re2dfa.c 1.5.2.6 Wed, 26 May 1999 15:55:50 -0700 cengiz $
  */
 /*
  * RSd release 0.1:
@@ -16,7 +16,7 @@
  *
  */
 
-#include "config.hh"
+#include "config.h"
 #include <stdio.h>
 #include <sys/types.h>
 #include <stdlib.h>
@@ -403,7 +403,7 @@ rd_insert_arc(rd_dq  *alist,		/* List of arcs */
      */
 
     arc = *arctoins; 		/* Struct copy */
-    if (RDQ_IS_EMPTY(alist)) {
+    if (RDQ_ISEMPTY(alist)) {
 	RD_ALLOC_ARC(nra, arc.ra_low, arc.ra_high);
 	RD_OR_BITS(nra->ra_mask, arc.ra_mask);
 	RDQ_INSERT_AFTER(alist, &(nra->ra_arcs));
@@ -593,20 +593,21 @@ rd_concatenate(rd_fm *fm1,	/* To this fm.... */
     int		fm2e;		/* Does fm2 accept the empty string? */
 
     if (fm1->eolexp) {
-       rd_free_nfa(fm2);
-       fm2 = NULL;
        if (!RD_ACCEPTS_EMPTY_STRING(fm2) || fm2->bolexp) {
 	  rd_free_nfa(fm1);
 	  fm1 = rd_empty_set_machine();
        }
+       rd_free_nfa(fm2);
+       fm2 = NULL;
        return fm1;
     }
     if (fm2->bolexp) {
-       rd_free_nfa(fm1);
        if (RD_ACCEPTS_EMPTY_STRING(fm1) && !fm1->eolexp) {
+	  rd_free_nfa(fm1);
 	  fm1 = fm2;
 	  fm2 = NULL;
        } else {
+	  rd_free_nfa(fm1);
 	  rd_free_nfa(fm2);
 	  fm2 = NULL;
 	  fm1 = rd_empty_set_machine();
@@ -716,7 +717,8 @@ rd_alternate(rd_fm *fm1,			/* To this NFA.. */
 rd_fm *
 rd_zero_or_one(rd_fm *fm1)
 {
-    RDQ_INSERT_BEFORE(&(fm1->rf_final), &(fm1->rf_start->rs_final));
+   if (RDQ_ISEMPTY(&(fm1->rf_start->rs_final)))
+      RDQ_INSERT_BEFORE(&(fm1->rf_final), &(fm1->rf_start->rs_final));
     return fm1;
 }
 
@@ -990,7 +992,7 @@ rd_ntod(rd_fm* nfa)
     RD_ALLOC_DFA_STATE(rej); 
     rjc = 0;
 
-    while (!RDQ_IS_EMPTY(&unp)) {
+    while (!RDQ_ISEMPTY(&unp)) {
 
 	/*
 	 * Remove first unprocessed state in list. Move it over
@@ -1225,6 +1227,9 @@ rd_recursively_mark(int i,
     rd_pair	*rp;		/* Current pair under consideration */
     rd_pair	*nrp;		/* Next pair */
 
+    if (RD_NOTSAME(i,j)->rt_mark)
+       return;
+
     RD_NOTSAME(i,j)->rt_mark = 1;
     rp = RD_NOTSAME(i,j)->rt_pair;
     while (rp) {
@@ -1465,7 +1470,7 @@ rd_print_nfa(rd_fm *fm)
     printf("\n\tARCS:\n");
     RDQ_LIST_START(&(fm->rf_states), fm, rs, rd_state) {
 	printf("\t\tFrom: %d\n", rs->rs_bit);
-	if (RDQ_IS_EMPTY(&(rs->rs_arcs))) {
+	if (RDQ_ISEMPTY(&(rs->rs_arcs))) {
 	    printf("\t\t\tNo outgoing arcs\n");
 	    continue;
 	}
@@ -1516,7 +1521,7 @@ rd_print_dfa(rd_fm *fm)
     printf("\n\tARCS:\n");
     RDQ_LIST_START(&(fm->rf_states), fm, rs, rd_state) {
 	printf("\t\tFrom: %d\n", rs->rs_bit);
-	if (RDQ_IS_EMPTY(&(rs->rs_arcs))) {
+	if (RDQ_ISEMPTY(&(rs->rs_arcs))) {
 	    printf("\t\t\tNo outgoing arcs\n");
 	    continue;
 	}
@@ -1592,11 +1597,11 @@ rd_fm *rd_empty_set_dfa() {
 }
 
 /*
- * rd_is_empty_dfa returns true if fm recognizes the empty language
+ * rd_isEmpty_dfa returns true if fm recognizes the empty language
  */
 
-int rd_is_empty_dfa(rd_fm *fm) {
-   return RDQ_IS_EMPTY(&(fm->rf_final));
+int rd_isEmpty_dfa(rd_fm *fm) {
+   return RDQ_ISEMPTY(&(fm->rf_final));
 }
 
 /*
@@ -1875,7 +1880,7 @@ int rd_equal_dfa(rd_fm *fm1, rd_fm *fm2) {
    fm3 = rd_duplicate_dfa(fm2);
    rd_complement_dfa(fm3);
    fm4 = rd_intersect_dfa(fm3, fm1);
-   result = rd_is_empty_dfa(fm4);
+   result = rd_isEmpty_dfa(fm4);
    rd_free_nfa(fm3);
    rd_free_nfa(fm4);
 
@@ -1885,7 +1890,7 @@ int rd_equal_dfa(rd_fm *fm1, rd_fm *fm2) {
    fm3 = rd_duplicate_dfa(fm1);
    rd_complement_dfa(fm3);
    fm4 = rd_intersect_dfa(fm3, fm2);
-   result = rd_is_empty_dfa(fm4);
+   result = rd_isEmpty_dfa(fm4);
    rd_free_nfa(fm3);
    rd_free_nfa(fm4);
 
@@ -1956,7 +1961,7 @@ rd_fm *rd_make_eol(rd_fm *fm) {
  * THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  * Questions concerning this software should be directed to 
- * info-ra@isi.edu.
+ * ratoolset@isi.edu.
  *
  */
 
