@@ -1,41 +1,38 @@
-// $Id: roe_tcl.cc 1.4.1.1 Mon, 18 Aug 1997 17:11:28 -0700 wlee $
-// 
+//  $Id: roe_tcl.cc 1.11 Wed, 10 Jun 1998 15:31:22 -0700 cengiz $
+//
 //  Copyright (c) 1994 by the University of Southern California
-//  and/or the International Business Machines Corporation.
 //  All rights reserved.
 //
-//  Permission to use, copy, modify, and distribute this software and
-//  its documentation in source and binary forms for lawful
-//  non-commercial purposes and without fee is hereby granted, provided
-//  that the above copyright notice appear in all copies and that both
-//  the copyright notice and this permission notice appear in supporting
-//  documentation, and that any documentation, advertising materials,
-//  and other materials related to such distribution and use acknowledge
-//  that the software was developed by the University of Southern
-//  California, Information Sciences Institute and/or the International
-//  Business Machines Corporation.  The name of the USC or IBM may not
-//  be used to endorse or promote products derived from this software
-//  without specific prior written permission.
+//  Permission to use, copy, modify, and distribute this software and its
+//  documentation in source and binary forms for lawful non-commercial
+//  purposes and without fee is hereby granted, provided that the above
+//  copyright notice appear in all copies and that both the copyright
+//  notice and this permission notice appear in supporting documentation,
+//  and that any documentation, advertising materials, and other materials
+//  related to such distribution and use acknowledge that the software was
+//  developed by the University of Southern California, Information
+//  Sciences Institute. The name of the USC may not be used to endorse or
+//  promote products derived from this software without specific prior
+//  written permission.
 //
-//  NEITHER THE UNIVERSITY OF SOUTHERN CALIFORNIA NOR INTERNATIONAL
-//  BUSINESS MACHINES CORPORATION MAKES ANY REPRESENTATIONS ABOUT
-//  THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.  THIS SOFTWARE IS
-//  PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES,
-//  INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
-//  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, TITLE, AND 
-//  NON-INFRINGEMENT.
+//  THE UNIVERSITY OF SOUTHERN CALIFORNIA DOES NOT MAKE ANY
+//  REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY
+//  PURPOSE.  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
+//  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
+//  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
+//  TITLE, AND NON-INFRINGEMENT.
 //
-//  IN NO EVENT SHALL USC, IBM, OR ANY OTHER CONTRIBUTOR BE LIABLE FOR ANY
-//  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, WHETHER IN CONTRACT,
-//  TORT, OR OTHER FORM OF ACTION, ARISING OUT OF OR IN CONNECTION WITH,
-//  THE USE OR PERFORMANCE OF THIS SOFTWARE.
+//  IN NO EVENT SHALL USC, OR ANY OTHER CONTRIBUTOR BE LIABLE FOR ANY
+//  SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, WHETHER IN CONTRACT, TORT,
+//  OR OTHER FORM OF ACTION, ARISING OUT OF OR IN CONNECTION WITH, THE USE
+//  OR PERFORMANCE OF THIS SOFTWARE.
 //
 //  Questions concerning this software should be directed to 
-//  info-ra@isi.edu.
+//  ratoolset@isi.edu.
 //
-//  Author(s): Cengiz Alaettinoglu <cengiz@isi.edu>
+//  Author(s): Cengiz Alaettinoglu <cengiz@ISI.EDU>
 
-#include "config.hh"
+#include "config.h"
 #include <cstdlib>
 #include <cstdarg>
 
@@ -55,10 +52,9 @@ extern "C" {
 #include "route-list.hh"
 #include "object.hh"
 #include "schedule.hh"
-#include "whois.hh"
+#include "irr/irr.hh"
 #include "roe_config.hh"
-#include "ASMap.h"
-#include "PrefaskMap.hh"
+#include <cstdarg>
 
 int roe_show_reg_no            = 1;
 int roe_show_reg_single        = 1;
@@ -85,6 +81,8 @@ Tk_Window w;                    /* The main window for the application.  If
 Tcl_Interp *interp;      /* Interpreter for this application. */
 
 #include <cstdarg>
+// Added by wlee@isi.edu
+extern IRR *whois;
 
 inline void tcl_error_handle() {
    if (*interp->result)
@@ -164,8 +162,8 @@ int roe_routes_hint(ClientData clientData, Tcl_Interp *interp,
     if ((roe_routes_hint_index != index) && (index >= 0)) {
       char buffer1[64], buffer2[64];
     //    Prefask_map(routelist(index)->route).get_address(buffer);
-      Prefask &prefask = Prefask_map(routelist(index)->route);
-      unsigned int prefix = prefask.get_prefix();
+      PrefixRange &prefask = routelist(index)->route;
+      unsigned int prefix = prefask.get_ipaddr();
       unsigned int mask   = prefask.get_mask();
       unsigned int prefix_max = prefix | ~mask;
       roe_routes_hint_index = index;
@@ -189,7 +187,7 @@ int roe_file_new_as(ClientData clientData, Tcl_Interp *interp,
       break;
     case 2:
       routelist.init();
-      routelist.load(argv[1]);
+      routelist.load(atoi(argv[1] + 2));
       routelist.display();
       break;
     default:
@@ -232,7 +230,9 @@ int roe_show_object(ClientData clientData, Tcl_Interp *interp,
       return TCL_ERROR;
    }
 
-   object.display(argv[1], argv[2], argv[3]);
+   // Modified by wlee@isi.edu
+   object.display(argv[1], atoi(argv[2]), argv[3]);
+   //   object.display(argv[1], argv[2], argv[3]);
 
    return TCL_OK;
 }
@@ -553,7 +553,7 @@ void tcl_link_vars() {
 }
 
 RouteList routelist;
-Object object;
+RoeObject object;
 Schedule schedule;
 
 void roe_initialize_GUI() {
@@ -627,7 +627,7 @@ void roe_initialize_GUI() {
    tcl_Eval("%s", layout_tk);
 #endif // LOAD_FILE
    char sources[256];
-   strncpy(sources, whois.GetSources(), sizeof(sources));
+   strncpy(sources, whois->GetSources(), sizeof(sources));
    for (char *s = strtok(sources, ","); s; s = strtok(NULL, ","))
       tcl_Eval(".menubar.selection.menu.sources add command -label %s -command { roe_del_selected_matching_source [.routes.list curselection] %s}", s, s);
 
@@ -657,7 +657,7 @@ int Tcl_AppInit(Tcl_Interp *interp) {
    }
 #endif
 
-#ifdef 0
+#ifdef NEVER
    if (roe_Init(interp) == TCL_ERROR) {
       return TCL_ERROR;
    }
@@ -665,4 +665,3 @@ int Tcl_AppInit(Tcl_Interp *interp) {
 
     return TCL_OK;
 }
-
